@@ -36,7 +36,6 @@
 
 #include <algorithm>
 
-#include "RequestGroup.h"
 #include "Request.h"
 #include "DownloadEngine.h"
 #include "Option.h"
@@ -47,6 +46,7 @@
 #include "DlAbortEx.h"
 #include "DlRetryEx.h"
 #include "DownloadFailureException.h"
+#include "CreateRequestCommand.h"
 #include "InitiateConnectionCommandFactory.h"
 #include "SleepCommand.h"
 #ifdef ENABLE_ASYNC_DNS
@@ -61,25 +61,29 @@
 #include "ServerStat.h"
 #include "RequestGroupMan.h"
 #include "A2STR.h"
-#include "Util.h"
+#include "util.h"
+#include "LogFactory.h"
+#include "DownloadContext.h"
 
 namespace aria2 {
 
 AbstractCommand::AbstractCommand(int32_t cuid,
 				 const SharedHandle<Request>& req,
+				 const SharedHandle<FileEntry>& fileEntry,
 				 RequestGroup* requestGroup,
 				 DownloadEngine* e,
 				 const SocketHandle& s):
-  Command(cuid), RequestGroupAware(requestGroup),
-  req(req), e(e), socket(s),
+  Command(cuid), _requestGroup(requestGroup),
+  req(req), _fileEntry(fileEntry), e(e), socket(s),
   checkSocketIsReadable(false), checkSocketIsWritable(false),
   nameResolverCheck(false)
 {
   if(!socket.isNull() && socket->isOpen()) {
     setReadCheckSocket(socket);
   }
-  timeout = this->e->option->getAsInt(PREF_TIMEOUT);
+  timeout = _requestGroup->getTimeout();
   _requestGroup->increaseStreamConnection();
+  _requestGroup->increaseNumCommand();
 }
 
 AbstractCommand::~AbstractCommand() {
@@ -88,6 +92,7 @@ AbstractCommand::~AbstractCommand() {
 #ifdef ENABLE_ASYNC_DNS
   disableNameResolverCheck(_asyncNameResolver);
 #endif // ENABLE_ASYNC_DNS
+  _requestGroup->decreaseNumCommand();
   _requestGroup->decreaseStreamConnection();
 }
 
@@ -99,16 +104,24 @@ bool AbstractCommand::execute() {
       //logger->debug("CUID#%d - finished.", cuid);
       return true;
     }
-    PeerStatHandle peerStat;
-    if(!_requestGroup->getSegmentMan().isNull()) {
-      peerStat = _requestGroup->getSegmentMan()->getPeerStat(cuid);
-    }
-    if(!peerStat.isNull()) {
-      if(peerStat->getStatus() == PeerStat::REQUEST_IDLE) {
-	logger->info(MSG_ABORT_REQUESTED, cuid);
-	onAbort();
-	req->resetUrl();
-	tryReserved();
+    // TODO it is not needed to check other PeerStats every time.
+    // Find faster Request when no segment is available.
+    if(!req.isNull() && _fileEntry->countPooledRequest() > 0 &&
+       !_requestGroup->getPieceStorage()->hasMissingUnusedPiece()) {
+      SharedHandle<Request> fasterRequest = _fileEntry->findFasterRequest(req);
+      if(!fasterRequest.isNull()) {
+	logger->info("CUID#%d - Use faster Request hostname=%s, port=%u",
+		     cuid,
+		     fasterRequest->getHost().c_str(),
+		     fasterRequest->getPort());
+
+	// Cancel current Request object and use faster one.
+	_fileEntry->removeRequest(req);
+	Command* command =
+	  InitiateConnectionCommandFactory::createInitiateConnectionCommand
+	  (cuid, fasterRequest, _fileEntry, _requestGroup, e);
+	e->setNoWait(true);
+	e->commands.push_back(command);
 	return true;
       }
     }
@@ -123,71 +136,117 @@ bool AbstractCommand::execute() {
       if(!_requestGroup->getPieceStorage().isNull()) {
 	_segments.clear();
 	_requestGroup->getSegmentMan()->getInFlightSegment(_segments, cuid);
-	while(_segments.size() < req->getMaxPipelinedRequest()) {
-	  SegmentHandle segment = _requestGroup->getSegmentMan()->getSegment(cuid);
-	  if(segment.isNull()) {
-	    break;
+	if(req.isNull() || req->getMaxPipelinedRequest() == 1 ||
+	   _requestGroup->getDownloadContext()->getFileEntries().size() == 1) {
+	  if(_segments.empty()) {
+	    SharedHandle<Segment> segment =
+	      _requestGroup->getSegmentMan()->getSegment(cuid);
+	    if(!segment.isNull()) {
+	      _segments.push_back(segment);
+	    }
+	  }
+	  if(_segments.empty()) {
+	    // TODO socket could be pooled here if pipelining is enabled...
+	    logger->info(MSG_NO_SEGMENT_AVAILABLE, cuid);
+	    // When all segments are ignored in SegmentMan, there are
+	    // no URIs available, so don't retry.
+	    if(_requestGroup->getSegmentMan()->allSegmentsIgnored()) {
+	      return true;
+	    } else {
+	      return prepareForRetry(1);
+	    }
+	  }
+	} else {
+	  size_t maxSegments = req->getMaxPipelinedRequest();
+	  if(_segments.size() < maxSegments) {
+	      _requestGroup->getSegmentMan()->getSegment
+	      (_segments, cuid, _fileEntry, maxSegments);
+	  }
+	  if(_segments.empty()) {
+	    return prepareForRetry(0);
 	  }
-	  _segments.push_back(segment);
-	}
-	if(_segments.empty()) {
-	  // TODO socket could be pooled here if pipelining is enabled...
-	  logger->info(MSG_NO_SEGMENT_AVAILABLE, cuid);
-	  return prepareForRetry(1);
 	}
       }
       return executeInternal();
     } else if(_errorEvent) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat(MSG_NETWORK_PROBLEM,
 		      socket->getSocketError().c_str()).str());
     } else {
       if(checkPoint.elapsed(timeout)) {
 	// timeout triggers ServerStat error state.
+
 	SharedHandle<ServerStat> ss =
 	  e->_requestGroupMan->getOrCreateServerStat(req->getHost(),
 						     req->getProtocol());
 	ss->setError();
 
-	throw DlRetryEx(EX_TIME_OUT);
+	throw DL_RETRY_EX2(EX_TIME_OUT, downloadresultcode::TIME_OUT);
       }
       e->commands.push_back(this);
       return false;
     }
   } catch(DlAbortEx& err) {
-    logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUrl().c_str());
+    if(req.isNull()) {
+      logger->debug(EX_EXCEPTION_CAUGHT, err);
+    } else {
+      logger->error(MSG_DOWNLOAD_ABORTED,
+		    DL_ABORT_EX2(StringFormat
+				 ("URI=%s", req->getCurrentUrl().c_str()).str(),err),
+		    cuid, req->getUrl().c_str());
+      _fileEntry->addURIResult(req->getUrl(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
+    }
     onAbort();
-    req->resetUrl();
     tryReserved();
     return true;
   } catch(DlRetryEx& err) {
-    logger->info(MSG_RESTARTING_DOWNLOAD, err, cuid, req->getUrl().c_str());
+    assert(!req.isNull());
+    logger->info(MSG_RESTARTING_DOWNLOAD,
+		 DL_RETRY_EX2(StringFormat
+			     ("URI=%s", req->getCurrentUrl().c_str()).str(),err),
+		 cuid, req->getUrl().c_str());
     req->addTryCount();
     req->resetRedirectCount();
-    bool isAbort = e->option->getAsInt(PREF_MAX_TRIES) != 0 &&
-      req->getTryCount() >= (unsigned int)e->option->getAsInt(PREF_MAX_TRIES);
+    const unsigned int maxTries = getOption()->getAsInt(PREF_MAX_TRIES);
+    bool isAbort = maxTries != 0 && req->getTryCount() >= maxTries;
     if(isAbort) {
       onAbort();
-    }
-    // In case where Request::getCurrentUrl() is not a valid URI.
-    req->resetUrl();
-    if(isAbort) {
       logger->info(MSG_MAX_TRY, cuid, req->getTryCount());
       logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUrl().c_str());
+      _fileEntry->addURIResult(req->getUrl(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
       tryReserved();
       return true;
     } else {
-      return prepareForRetry(e->option->getAsInt(PREF_RETRY_WAIT));
+      return prepareForRetry(0);
     }
   } catch(DownloadFailureException& err) {
     logger->error(EX_EXCEPTION_CAUGHT, err);
+    if(!req.isNull()) {
+      _fileEntry->addURIResult(req->getUrl(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
+    }
     _requestGroup->setHaltRequested(true);
     return true;
   }
 }
 
 void AbstractCommand::tryReserved() {
-  _requestGroup->removeServerHost(cuid);
+  if(_requestGroup->getDownloadContext()->getFileEntries().size() == 1) {
+    const SharedHandle<FileEntry>& entry =
+      _requestGroup->getDownloadContext()->getFirstFileEntry();
+    // Don't create new command if currently file length is unknown
+    // and there are no URI left. Because file length is unknown, we
+    // can assume that there are no in-flight request object.
+    if(entry->getLength() == 0 && entry->getRemainingUris().empty()) {
+      logger->debug("CUID#%d - Not trying next request."
+		    " No reserved/pooled request is remaining and"
+		    " total length is still unknown.", cuid);
+      return;
+    }
+  }
+  logger->debug("CUID#%d - Trying reserved/pooled request.", cuid);
   Commands commands;
   _requestGroup->createNextCommand(commands, e, 1);
   e->setNoWait(true);
@@ -198,7 +257,16 @@ bool AbstractCommand::prepareForRetry(time_t wait) {
   if(!_requestGroup->getPieceStorage().isNull()) {
     _requestGroup->getSegmentMan()->cancelSegment(cuid);
   }
-  Command* command = InitiateConnectionCommandFactory::createInitiateConnectionCommand(cuid, req, _requestGroup, e);
+  if(!req.isNull()) {
+    _fileEntry->poolRequest(req);
+    logger->debug("CUID#%d - Pooling request URI=%s",
+		  cuid, req->getUrl().c_str());
+    if(!_requestGroup->getSegmentMan().isNull()) {
+      _requestGroup->getSegmentMan()->recognizeSegmentFor(_fileEntry);
+    }
+  }
+
+  Command* command = new CreateRequestCommand(cuid, _requestGroup, e);
   if(wait == 0) {
     e->setNoWait(true);
     e->commands.push_back(command);
@@ -211,12 +279,18 @@ bool AbstractCommand::prepareForRetry(time_t wait) {
 }
 
 void AbstractCommand::onAbort() {
-  logger->debug(MSG_UNREGISTER_CUID, cuid);
-  //_segmentMan->unregisterId(cuid);
+  if(!req.isNull()) {
+    // TODO This might be a problem if the failure is caused by proxy.
+    e->_requestGroupMan->getOrCreateServerStat(req->getHost(),
+					       req->getProtocol())->setError();
+    _fileEntry->removeIdenticalURI(req->getUrl());
+    _fileEntry->removeRequest(req);
+  }
+
+  logger->debug("CUID#%d - Aborting download", cuid);
   if(!_requestGroup->getPieceStorage().isNull()) {
     _requestGroup->getSegmentMan()->cancelSegment(cuid);
   }
-  _requestGroup->removeIdenticalURI(req->getUrl());
 }
 
 void AbstractCommand::disableReadCheckSocket() {
@@ -292,7 +366,7 @@ void AbstractCommand::setWriteCheckSocketIf
 }
 
 static const std::string& getProxyStringFor(const std::string& proxyPref,
-					    const Option* option)
+					    const SharedHandle<Option>& option)
 {
   if(option->defined(proxyPref)) {
     return option->get(proxyPref);
@@ -302,7 +376,7 @@ static const std::string& getProxyStringFor(const std::string& proxyPref,
 }
 
 static bool isProxyUsed(const std::string& proxyPref,
-			const Option* option)
+			const SharedHandle<Option>& option)
 {
   std::string proxy = getProxyStringFor(proxyPref, option);
   if(proxy.empty()) {
@@ -312,7 +386,8 @@ static bool isProxyUsed(const std::string& proxyPref,
   }
 }
 
-static bool isProxyRequest(const std::string& protocol, const Option* option)
+static bool isProxyRequest(const std::string& protocol,
+			   const SharedHandle<Option>& option)
 {
   return
     (protocol == Request::PROTO_HTTP && isProxyUsed(PREF_HTTP_PROXY, option)) ||
@@ -320,14 +395,6 @@ static bool isProxyRequest(const std::string& protocol, const Option* option)
     (protocol == Request::PROTO_FTP && isProxyUsed(PREF_FTP_PROXY, option));
 }
 
-static bool isProxyGETRequest(const std::string& protocol, const Option* option)
-{
-  if(option->get(PREF_PROXY_METHOD) != V_GET) {
-    return false;
-  }
-  return isProxyRequest(protocol, option);
-}
-
 class DomainMatch {
 private:
   std::string _hostname;
@@ -336,10 +403,10 @@ public:
 
   bool operator()(const std::string& domain) const
   {
-    if(Util::startsWith(domain, ".")) {
-      return Util::endsWith(_hostname, domain);
+    if(util::startsWith(domain, ".")) {
+      return util::endsWith(_hostname, domain);
     } else {
-      return Util::endsWith(_hostname, "."+domain);
+      return util::endsWith(_hostname, "."+domain);
     }
   }
 };
@@ -347,8 +414,8 @@ public:
 static bool inNoProxy(const SharedHandle<Request>& req,
 		      const std::string& noProxy)
 {
-  std::deque<std::string> entries;
-  Util::slice(entries, noProxy, ',', true);
+  std::vector<std::string> entries;
+  util::split(noProxy, std::back_inserter(entries), ",", true);
   if(entries.empty()) {
     return false;
   }
@@ -359,12 +426,12 @@ static bool inNoProxy(const SharedHandle<Request>& req,
 
 bool AbstractCommand::isProxyDefined() const
 {
-  return isProxyRequest(req->getProtocol(), e->option) &&
-    !inNoProxy(req, e->option->get(PREF_NO_PROXY));
+  return isProxyRequest(req->getProtocol(), getOption()) &&
+    !inNoProxy(req, getOption()->get(PREF_NO_PROXY));
 }
 
 static const std::string& getProxyString(const SharedHandle<Request>& req,
-					 const Option* option)
+					 const SharedHandle<Option>& option)
 {
   if(req->getProtocol() == Request::PROTO_HTTP) {
     return getProxyStringFor(PREF_HTTP_PROXY, option);
@@ -380,10 +447,10 @@ static const std::string& getProxyString(const SharedHandle<Request>& req,
 SharedHandle<Request> AbstractCommand::createProxyRequest() const
 {
   SharedHandle<Request> proxyRequest;
-  if(inNoProxy(req, e->option->get(PREF_NO_PROXY))) {
+  if(inNoProxy(req, getOption()->get(PREF_NO_PROXY))) {
     return proxyRequest;
   }
-  std::string proxy = getProxyString(req, e->option);
+  std::string proxy = getProxyString(req, getOption());
   if(!proxy.empty()) {
     proxyRequest.reset(new Request());
     if(proxyRequest->setUrl(proxy)) {
@@ -417,11 +484,11 @@ bool AbstractCommand::asyncResolveHostname()
   case AsyncNameResolver::STATUS_SUCCESS:
     return true;
   case AsyncNameResolver::STATUS_ERROR:
-    if(!isProxyRequest(req->getProtocol(), e->option)) {
+    if(!isProxyRequest(req->getProtocol(), getOption())) {
       e->_requestGroupMan->getOrCreateServerStat
 	(req->getHost(), req->getProtocol())->setError();
     }
-    throw DlAbortEx(StringFormat(MSG_NAME_RESOLUTION_FAILED, cuid,
+    throw DL_ABORT_EX(StringFormat(MSG_NAME_RESOLUTION_FAILED, cuid,
 				 _asyncNameResolver->getHostname().c_str(),
 				 _asyncNameResolver->getError().c_str()).str());
   default:
@@ -459,7 +526,8 @@ bool AbstractCommand::nameResolveFinished() const {
 
 void AbstractCommand::prepareForNextAction(Command* nextCommand)
 {
-  CheckIntegrityEntryHandle entry(new StreamCheckIntegrityEntry(req, _requestGroup, nextCommand));
+  CheckIntegrityEntryHandle entry
+    (new StreamCheckIntegrityEntry(_requestGroup, nextCommand));
 
   std::deque<Command*> commands;
   _requestGroup->processCheckIntegrityEntry(commands, entry, e);
@@ -468,21 +536,55 @@ void AbstractCommand::prepareForNextAction(Command* nextCommand)
   e->setNoWait(true);
 }
 
-void AbstractCommand::checkIfConnectionEstablished
-(const SharedHandle<SocketCore>& socket)
+bool AbstractCommand::checkIfConnectionEstablished
+(const SharedHandle<SocketCore>& socket,
+ const std::string& connectedHostname,
+ const std::string& connectedAddr,
+ uint16_t connectedPort)
 {
   if(socket->isReadable(0)) {
     std::string error = socket->getSocketError();
     if(!error.empty()) {
+      // See also InitiateConnectionCommand::executeInternal()
+      e->markBadIPAddress(connectedHostname, connectedAddr, connectedPort);
+      if(!e->findCachedIPAddress(connectedHostname, connectedPort).empty()) {
+	logger->info(MSG_CONNECT_FAILED_AND_RETRY,
+		     cuid, connectedAddr.c_str(), connectedPort);
+	Command* command =
+	  InitiateConnectionCommandFactory::createInitiateConnectionCommand
+	  (cuid, req, _fileEntry, _requestGroup, e);
+	e->setNoWait(true);
+	e->commands.push_back(command);
+	return false;
+      }
+      e->removeCachedIPAddress(connectedHostname, connectedPort);
       // Don't set error if proxy server is used and its method is GET.
-      if(!isProxyGETRequest(req->getProtocol(), e->option)) {
+      if(resolveProxyMethod(req->getProtocol()) != V_GET ||
+	 !isProxyRequest(req->getProtocol(), getOption())) {
 	e->_requestGroupMan->getOrCreateServerStat
 	  (req->getHost(), req->getProtocol())->setError();
       }
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat(MSG_ESTABLISHING_CONNECTION_FAILED, error.c_str()).str());
     }
   }
+  return true;
+}
+
+const std::string& AbstractCommand::resolveProxyMethod
+(const std::string& protocol) const
+{
+  if(getOption()->get(PREF_PROXY_METHOD) == V_TUNNEL ||
+     Request::PROTO_HTTPS == protocol) {
+    return V_TUNNEL;
+  } else {
+    return V_GET;
+  }
+}
+
+const SharedHandle<Option>& AbstractCommand::getOption() const
+{
+  return _requestGroup->getOption();
 }
 
 } // namespace aria2
