@@ -35,15 +35,26 @@
 #include "AbstractSingleDiskAdaptor.h"
 #include "File.h"
 #include "SingleFileAllocationIterator.h"
+#ifdef HAVE_POSIX_FALLOCATE
+# include "FallocFileAllocationIterator.h"
+#endif // HAVE_POSIX_FALLOCATE
+#include "DiskWriter.h"
+
+namespace aria2 {
+
+AbstractSingleDiskAdaptor::AbstractSingleDiskAdaptor():
+  totalLength(0), _readOnly(false) {}
+
+AbstractSingleDiskAdaptor::~AbstractSingleDiskAdaptor() {}
 
 void AbstractSingleDiskAdaptor::initAndOpenFile()
 {
-  diskWriter->initAndOpenFile(getFilePath(), totalLength);
+  diskWriter->initAndOpenFile(totalLength);
 }
 
 void AbstractSingleDiskAdaptor::openFile()
 {
-  diskWriter->openFile(getFilePath(), totalLength);
+  diskWriter->openFile(totalLength);
 }
 
 void AbstractSingleDiskAdaptor::closeFile()
@@ -53,15 +64,15 @@ void AbstractSingleDiskAdaptor::closeFile()
 
 void AbstractSingleDiskAdaptor::openExistingFile()
 {
-  diskWriter->openExistingFile(getFilePath(), totalLength);
+  diskWriter->openExistingFile(totalLength);
 }
 
-void AbstractSingleDiskAdaptor::writeData(const unsigned char* data, int32_t len, int64_t offset)
+void AbstractSingleDiskAdaptor::writeData(const unsigned char* data, size_t len, off_t offset)
 {
   diskWriter->writeData(data, len, offset);
 }
 
-int32_t AbstractSingleDiskAdaptor::readData(unsigned char* data, int32_t len, int64_t offset)
+ssize_t AbstractSingleDiskAdaptor::readData(unsigned char* data, size_t len, off_t offset)
 {
   return diskWriter->readData(data, len, offset);
 }
@@ -71,14 +82,31 @@ bool AbstractSingleDiskAdaptor::fileExists()
   return File(getFilePath()).exists();
 }
 
+uint64_t AbstractSingleDiskAdaptor::size()
+{
+  return File(getFilePath()).size();
+}
+
+void AbstractSingleDiskAdaptor::truncate(uint64_t length)
+{
+  diskWriter->truncate(length);
+}
+
 FileAllocationIteratorHandle AbstractSingleDiskAdaptor::fileAllocationIterator()
 {
-  SingleFileAllocationIteratorHandle h =
-    new SingleFileAllocationIterator(this,
-				     size(),
-				     totalLength);
-  h->init();
-  return h;
+#ifdef HAVE_POSIX_FALLOCATE
+  if(_fallocate) {
+    SharedHandle<FallocFileAllocationIterator> h
+      (new FallocFileAllocationIterator(this, size(), totalLength));
+    return h;
+  } else
+#endif // HAVE_POSIX_FALLOCATE
+    {
+      SingleFileAllocationIteratorHandle h
+	(new SingleFileAllocationIterator(this, size(), totalLength));
+      h->init();
+      return h;
+    }
 }
 
 void AbstractSingleDiskAdaptor::enableDirectIO()
@@ -90,3 +118,34 @@ void AbstractSingleDiskAdaptor::disableDirectIO()
 {
   diskWriter->disableDirectIO();
 }
+
+void AbstractSingleDiskAdaptor::enableReadOnly()
+{
+  diskWriter->enableReadOnly();
+  _readOnly = true;
+}
+
+void AbstractSingleDiskAdaptor::disableReadOnly()
+{
+  diskWriter->disableReadOnly();
+  _readOnly = false;
+}
+
+void AbstractSingleDiskAdaptor::cutTrailingGarbage()
+{
+  if(File(getFilePath()).size() > totalLength) {
+    diskWriter->truncate(totalLength);
+  }
+}
+  
+void AbstractSingleDiskAdaptor::setDiskWriter(const DiskWriterHandle& diskWriter)
+{
+  this->diskWriter = diskWriter;
+}
+
+void AbstractSingleDiskAdaptor::setTotalLength(const uint64_t& totalLength)
+{
+  this->totalLength = totalLength;
+}
+
+} // namespace aria2
