@@ -52,7 +52,7 @@
 #include "DefaultDiskWriterFactory.h"
 #include "FileEntry.h"
 #include "DlAbortEx.h"
-#include "Util.h"
+#include "util.h"
 #include "a2functional.h"
 #include "Option.h"
 #include "StringFormat.h"
@@ -63,7 +63,7 @@
 namespace aria2 {
 
 DefaultPieceStorage::DefaultPieceStorage
-(const DownloadContextHandle& downloadContext, const Option* option):
+(const SharedHandle<DownloadContext>& downloadContext, const Option* option):
   downloadContext(downloadContext),
   bitfieldMan(BitfieldManFactory::getFactoryInstance()->
 	      createBitfieldMan(downloadContext->getPieceLength(),
@@ -261,10 +261,17 @@ SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
 
 #endif // ENABLE_BITTORRENT
 
-PieceHandle DefaultPieceStorage::getMissingPiece()
+bool DefaultPieceStorage::hasMissingUnusedPiece()
 {
   size_t index;
-  if(bitfieldMan->getSparseMissingUnusedIndex(index)) {
+  return bitfieldMan->getFirstMissingUnusedIndex(index);
+}
+
+PieceHandle DefaultPieceStorage::getSparseMissingUnusedPiece
+(const unsigned char* ignoreBitfield, size_t length)
+{
+  size_t index;
+  if(bitfieldMan->getSparseMissingUnusedIndex(index, ignoreBitfield, length)) {
     return checkOutPiece(index);
   } else {
     return SharedHandle<Piece>();
@@ -345,7 +352,6 @@ void DefaultPieceStorage::completePiece(const PieceHandle& piece)
   addPieceStats(piece->getIndex());
   if(downloadFinished()) {
     downloadContext->resetDownloadStopTime();
-    diskAdaptor->onDownloadComplete();
     if(isSelectiveDownloadingMode()) {
       logger->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
       // following line was commented out in order to stop sending request
@@ -420,10 +426,10 @@ size_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
 // not unittested
 void DefaultPieceStorage::setupFileFilter()
 {
-  std::deque<SharedHandle<FileEntry> > fileEntries =
+  const std::vector<SharedHandle<FileEntry> >& fileEntries =
     downloadContext->getFileEntries();
   bool allSelected = true;
-  for(std::deque<SharedHandle<FileEntry> >::const_iterator i =
+  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
 	fileEntries.begin(); i != fileEntries.end(); ++i) {
     if(!(*i)->isRequested()) {
       allSelected = false;
@@ -433,7 +439,7 @@ void DefaultPieceStorage::setupFileFilter()
   if(allSelected) {
     return;
   }
-  for(std::deque<SharedHandle<FileEntry> >::const_iterator i =
+  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
 	fileEntries.begin(); i != fileEntries.end(); ++i) {
     if((*i)->isRequested()) {
       bitfieldMan->addFilter((*i)->getOffset(), (*i)->getLength());
@@ -464,11 +470,12 @@ bool DefaultPieceStorage::allDownloadFinished()
 // not unittested
 void DefaultPieceStorage::initStorage()
 {
-  if(downloadContext->getFileMode() == DownloadContext::SINGLE) {
+  if(downloadContext->getFileEntries().size() == 1) {
     logger->debug("Instantiating DirectDiskAdaptor");
     DirectDiskAdaptorHandle directDiskAdaptor(new DirectDiskAdaptor());
     directDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
-    directDiskAdaptor->setFileEntries(downloadContext->getFileEntries());
+    directDiskAdaptor->setFileEntries(downloadContext->getFileEntries().begin(),
+				      downloadContext->getFileEntries().end());
 
     DiskWriterHandle writer =
       _diskWriterFactory->newDiskWriter(directDiskAdaptor->getFilePath());
@@ -479,10 +486,10 @@ void DefaultPieceStorage::initStorage()
     directDiskAdaptor->setDiskWriter(writer);
     this->diskAdaptor = directDiskAdaptor;
   } else {
-    // file mode == DownloadContext::MULTI
     logger->debug("Instantiating MultiDiskAdaptor");
     MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
-    multiDiskAdaptor->setFileEntries(downloadContext->getFileEntries());
+    multiDiskAdaptor->setFileEntries(downloadContext->getFileEntries().begin(),
+				     downloadContext->getFileEntries().end());
     if(option->getAsBool(PREF_ENABLE_DIRECT_IO)) {
       multiDiskAdaptor->allowDirectIO();
     }
