@@ -34,58 +34,45 @@
 /* copyright --> */
 #include "NameResolver.h"
 
-#ifdef ENABLE_ASYNC_DNS
-
-#ifdef HAVE_LIBCARES1_5
-void callback(void* arg, int status, int timeouts, struct hostent* host) {
-#else
-void callback(void* arg, int status, struct hostent* host) {
-#endif // HAVE_LIBCARES1_5
-  NameResolver* resolverPtr = (NameResolver*)arg;
-#ifdef HAVE_LIBARES
-  // This block is required since the assertion in ares_strerror fails
-  // if status = ARES_EDESTRUCTION is passed to ares_strerror as 1st argument.
-  // This does not happen in c-ares.
-  if(status == ARES_EDESTRUCTION) {
-    // we simply return in this case.
-    return;
-  }
-#endif
-  if(status != ARES_SUCCESS) {
-#ifdef HAVE_LIBCARES
-    resolverPtr->error = ares_strerror(status);
-#else
-    resolverPtr->error = ares_strerror(status, 0);
-#endif // HAVE_LIBCARES
-    resolverPtr->status = NameResolver::STATUS_ERROR;
-    return;
-  }
-  memcpy(&resolverPtr->addr, *host->h_addr_list, sizeof(struct in_addr));
-  resolverPtr->status = NameResolver::STATUS_SUCCESS;
-}
-
-#else // ENABLE_ASYNC_DNS
+#include <cstring>
 
 #include "DlAbortEx.h"
 #include "message.h"
+#include "StringFormat.h"
+#include "util.h"
 
-void NameResolver::resolve(const string& hostname)
+namespace aria2 {
+
+NameResolver::NameResolver():_socktype(0), _family(AF_UNSPEC) {}
+
+void NameResolver::resolve(std::deque<std::string>& resolvedAddresses,
+			   const std::string& hostname)
 {
-  memset(&_addr, 0, sizeof(in_addr));
-  struct addrinfo ai;
-  memset((char*)&ai, 0, sizeof(ai));
-  ai.ai_flags = 0;
-  ai.ai_family = PF_INET;
-  ai.ai_socktype = SOCK_STREAM;
-  ai.ai_protocol = 0; 
+  struct addrinfo hints;
   struct addrinfo* res;
-  int ec;
-  if((ec = getaddrinfo(hostname.c_str(), 0, &ai, &res)) != 0) {
-    throw new DlAbortEx(EX_RESOLVE_HOSTNAME,
-			hostname.c_str(), gai_strerror(ec));
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family = _family;
+  hints.ai_socktype = _socktype;
+  hints.ai_flags = 0;
+  hints.ai_protocol = 0;
+  int s;
+  s = getaddrinfo(hostname.c_str(), 0, &hints, &res);
+  if(s) {
+    throw DL_ABORT_EX(StringFormat(EX_RESOLVE_HOSTNAME,
+				 hostname.c_str(), gai_strerror(s)).str());
+  }
+  struct addrinfo* rp;
+  for(rp = res; rp; rp = rp->ai_next) {
+    std::pair<std::string, uint16_t> addressPort
+      = util::getNumericNameInfo(rp->ai_addr, rp->ai_addrlen);
+    resolvedAddresses.push_back(addressPort.first);
   }
-  _addr = ((struct sockaddr_in*)res->ai_addr)->sin_addr;
   freeaddrinfo(res);
 }
 
-#endif // ENABLE_ASYNC_DNS
+void NameResolver::setSocktype(int socktype)
+{
+  _socktype = socktype;
+}
+
+} // namespace aria2
