@@ -45,7 +45,7 @@
 #include <iterator>
 
 #include "NameMatchOptionHandler.h"
-#include "Util.h"
+#include "util.h"
 #include "DlAbortEx.h"
 #include "prefs.h"
 #include "Option.h"
@@ -53,6 +53,9 @@
 #include "A2STR.h"
 #include "Request.h"
 #include "a2functional.h"
+#include "message.h"
+#include "File.h"
+#include "FileEntry.h"
 
 namespace aria2 {
 
@@ -157,15 +160,15 @@ public:
 
   virtual void parseArg(Option& option, const std::string& optarg)
   {
-    IntSequence seq = Util::parseIntRange(optarg);
+    IntSequence seq = util::parseIntRange(optarg);
     while(seq.hasNext()) {
       int32_t v = seq.next();
       if(v < _min || _max < v) {
 	std::string msg = _optName;
 	strappend(msg, " ", _("must be between %s and %s."));
 	throw DL_ABORT_EX
-	  (StringFormat(msg.c_str(), Util::itos(_min).c_str(),
-			Util::itos(_max).c_str()).str());
+	  (StringFormat(msg.c_str(), util::itos(_min).c_str(),
+			util::itos(_max).c_str()).str());
       }
       option.put(_optName, optarg);
     }
@@ -173,7 +176,7 @@ public:
 
   virtual std::string createPossibleValuesString() const
   {
-    return Util::itos(_min)+"-"+Util::itos(_max);
+    return util::itos(_min)+"-"+util::itos(_max);
   }
 };
 
@@ -196,26 +199,26 @@ public:
 
   virtual void parseArg(Option& option, const std::string& optarg)
   {
-    int64_t num = Util::parseLLInt(optarg);
+    int64_t num = util::parseLLInt(optarg);
     parseArg(option, num);
   }
 
   void parseArg(Option& option, int64_t number)
   {
     if((_min == -1 || _min <= number) && (_max ==  -1 || number <= _max)) {
-      option.put(_optName, Util::itos(number));
+      option.put(_optName, util::itos(number));
     } else {
       std::string msg = _optName;
       msg += " ";
       if(_min == -1 && _max != -1) {
 	msg += StringFormat(_("must be smaller than or equal to %s."),
-			    Util::itos(_max).c_str()).str();
+			    util::itos(_max).c_str()).str();
       } else if(_min != -1 && _max != -1) {
 	msg += StringFormat(_("must be between %s and %s."),
-			    Util::itos(_min).c_str(), Util::itos(_max).c_str()).str();
+			    util::itos(_min).c_str(), util::itos(_max).c_str()).str();
       } else if(_min != -1 && _max == -1) {
 	msg += StringFormat(_("must be greater than or equal to %s."),
-			    Util::itos(_min).c_str()).str();
+			    util::itos(_min).c_str()).str();
       } else {
 	msg += _("must be a number.");
       }
@@ -229,13 +232,13 @@ public:
     if(_min == -1) {
       values += "*";
     } else {
-      values += Util::itos(_min);
+      values += util::itos(_min);
     }
     values += "-";
     if(_max == -1) {
       values += "*";
     } else {
-      values += Util::itos(_max);
+      values += util::itos(_max);
     }
     return values;
   }
@@ -256,7 +259,7 @@ public:
 
   virtual void parseArg(Option& option, const std::string& optarg)
   {
-    int64_t num = Util::getRealSize(optarg);
+    int64_t num = util::getRealSize(optarg);
     NumberOptionHandler::parseArg(option, num);
   }
 };
@@ -398,7 +401,7 @@ public:
   virtual void parseArg(Option& option, const std::string& optarg)
   {
     // See optarg is in the fomrat of "INDEX=PATH"
-    Util::parseIndexPath(optarg);
+    util::parseIndexPath(optarg);
     std::string value = option.get(_optName);
     strappend(value, optarg, "\n");
     option.put(_optName, value);
@@ -490,7 +493,7 @@ public:
     std::stringstream s;
     std::copy(_validParamValues.begin(), _validParamValues.end(),
 	      std::ostream_iterator<std::string>(s, ","));
-    return Util::trim(s.str(), ", ");
+    return util::trim(s.str(), ", ");
   }
 };
 
@@ -515,8 +518,8 @@ public:
 
   virtual void parseArg(Option& option, const std::string& optarg)
   {
-    std::pair<std::string, std::string> proxy = Util::split(optarg, ":");
-    int32_t port = Util::parseInt(proxy.second);
+    std::pair<std::string, std::string> proxy = util::split(optarg, ":");
+    int32_t port = util::parseInt(proxy.second);
     if(proxy.first.empty() || proxy.second.empty() ||
        port <= 0 || 65535 < port) {
       throw DL_ABORT_EX(_("unrecognized proxy format"));
@@ -528,7 +531,7 @@ public:
   void setHostAndPort(Option& option, const std::string& hostname, uint16_t port)
   {
     option.put(_hostOptionName, hostname);
-    option.put(_portOptionName, Util::uitos(port));
+    option.put(_portOptionName, util::uitos(port));
   }
 
   virtual std::string createPossibleValuesString() const
@@ -554,7 +557,7 @@ public:
   {
     Request req;
     std::string url;
-    if(Util::startsWith(optarg, "http://")) {
+    if(util::startsWith(optarg, "http://")) {
       url = optarg;
     } else {
       url = "http://"+optarg;
@@ -572,6 +575,59 @@ public:
   }
 };
 
+class LocalFilePathOptionHandler : public NameMatchOptionHandler {
+public:
+  LocalFilePathOptionHandler
+  (const std::string& optName,
+   const std::string& description = NO_DESCRIPTION,
+   const std::string& defaultValue = NO_DEFAULT_VALUE,
+   char shortName = 0):
+    NameMatchOptionHandler(optName, description, defaultValue,
+			   OptionHandler::REQ_ARG,
+			   shortName) {}
+
+  virtual void parseArg(Option& option, const std::string& optarg)
+  {
+    File f(optarg);
+    if(!f.exists() || f.isDir()) {
+      throw DL_ABORT_EX
+	(StringFormat(MSG_NOT_FILE, optarg.c_str()).str());
+    }
+    option.put(_optName, optarg);
+  }
+  
+  virtual std::string createPossibleValuesString() const
+  {
+    return "/path/to/file";
+  }
+};
+
+class PrioritizePieceOptionHandler:public NameMatchOptionHandler {
+public:
+  PrioritizePieceOptionHandler
+  (const std::string& optName,
+   const std::string& description = NO_DESCRIPTION,
+   const std::string& defaultValue = NO_DEFAULT_VALUE,
+   char shortName = 0):
+    NameMatchOptionHandler(optName, description, defaultValue,
+			   OptionHandler::REQ_ARG, shortName) {}
+
+  virtual void parseArg(Option& option, const std::string& optarg)
+  {
+    // Parse optarg against empty FileEntry list to detect syntax
+    // error.
+    std::vector<size_t> result;
+    util::parsePrioritizePieceRange
+      (result, optarg, std::vector<SharedHandle<FileEntry> >(), 1024);
+    option.put(_optName, optarg);
+  }
+
+  virtual std::string createPossibleValuesString() const
+  {
+    return "head[=SIZE],tail[=SIZE]";
+  }
+};
+
 } // namespace aria2
 
 #endif // _D_OPTION_HANDLER_IMPL_H_
