@@ -63,7 +63,7 @@
 #include "Option.h"
 #include "prefs.h"
 #include "File.h"
-#include "Util.h"
+#include "util.h"
 #include "Command.h"
 #include "FileEntry.h"
 
@@ -72,7 +72,7 @@ namespace aria2 {
 RequestGroupMan::RequestGroupMan(const RequestGroups& requestGroups,
 				 unsigned int maxSimultaneousDownloads,
 				 const Option* option):
-  _requestGroups(requestGroups),
+  _reservedGroups(requestGroups),
   _logger(LogFactory::getInstance()),
   _maxSimultaneousDownloads(maxSimultaneousDownloads),
   _gidCounter(0),
@@ -80,13 +80,14 @@ RequestGroupMan::RequestGroupMan(const RequestGroups& requestGroups,
   _serverStatMan(new ServerStatMan()),
   _maxOverallDownloadSpeedLimit
   (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)),
-  _maxOverallUploadSpeedLimit(option->getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT))
+  _maxOverallUploadSpeedLimit(option->getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT)),
+  _xmlRpc(option->getAsBool(PREF_ENABLE_XML_RPC))
 {}
 
 bool RequestGroupMan::downloadFinished()
 {
 #ifdef ENABLE_XML_RPC
-  if(_option->getAsBool(PREF_ENABLE_XML_RPC)) {
+  if(_xmlRpc) {
     return false;
   }
 #endif // ENABLE_XML_RPC
@@ -203,9 +204,10 @@ static void executeHook(const std::string& command, int gid)
     LogFactory::getInstance()->error("fork() failed."
 				     " Cannot execute user command.");
   } else if(cpid == 0) {
-    execl(command.c_str(), command.c_str(), Util::itos(gid).c_str(), (char*)0);
+    execl(command.c_str(), command.c_str(), util::itos(gid).c_str(),
+	  reinterpret_cast<char*>(0));
     perror(("Could not execute user command: "+command).c_str());
-    _exit(1);
+    exit(EXIT_FAILURE);
   }
 #else
   PROCESS_INFORMATION pi;
@@ -217,7 +219,7 @@ static void executeHook(const std::string& command, int gid)
   memset(&pi, 0, sizeof (pi));
 
   std::string cmdline = command;
-  strappend(cmdline, " ", Util::itos(gid));
+  strappend(cmdline, " ", util::itos(gid));
 
   DWORD rc = CreateProcess(
     NULL,
@@ -248,10 +250,10 @@ static void executeStartHook
 static void executeStopHook
 (const SharedHandle<DownloadResult>& result, const Option* option)
 {
-  if(result->result == DownloadResult::FINISHED &&
+  if(result->result == downloadresultcode::FINISHED &&
      !option->blank(PREF_ON_DOWNLOAD_COMPLETE)) {
     executeHook(option->get(PREF_ON_DOWNLOAD_COMPLETE), result->gid);
-  } else if(result->result != DownloadResult::IN_PROGRESS &&
+  } else if(result->result != downloadresultcode::IN_PROGRESS &&
 	    !option->blank(PREF_ON_DOWNLOAD_ERROR)) {
     executeHook(option->get(PREF_ON_DOWNLOAD_ERROR), result->gid);
   } else if(!option->blank(PREF_ON_DOWNLOAD_STOP)) {
@@ -273,7 +275,7 @@ private:
     if(!sig.isNull() && !sig->getBody().empty()) {
       // filename of signature file is the path to download file followed by
       // ".sig".
-      std::string signatureFile = group->getFilePath()+".sig";
+      std::string signatureFile = group->getFirstFilePath()+".sig";
       if(sig->save(signatureFile)) {
 	_logger->notice(MSG_SIGNATURE_SAVED, signatureFile.c_str());
       } else {
@@ -300,10 +302,10 @@ public:
 	  group->applyLastModifiedTimeToLocalFiles();
 	  group->reportDownloadFinished();
 	  if(group->allDownloadFinished()) {
-	    group->getProgressInfoFile()->removeFile();
+	    group->removeControlFile();
 	    saveSignature(group);
 	  } else {
-	    group->getProgressInfoFile()->save();
+	    group->saveControlFile();
 	  }
 	  RequestGroups nextGroups;
 	  group->postDownloadProcessing(nextGroups);
@@ -315,7 +317,7 @@ public:
 				   nextGroups.begin(), nextGroups.end());
 	  }
 	} else {
-	  group->getProgressInfoFile()->save();
+	  group->saveControlFile();
 	}
       } catch(RecoverableException& ex) {
 	_logger->error(EX_EXCEPTION_CAUGHT, ex);
@@ -341,9 +343,10 @@ public:
       // Collect statistics during download in PeerStats and update/register
       // ServerStatMan
       if(!group->getSegmentMan().isNull()) {
+	bool singleConnection =
+	  group->getSegmentMan()->getPeerStats().size() == 1;
 	const std::deque<SharedHandle<PeerStat> >& peerStats =
-	  group->getSegmentMan()->getPeerStats();
-
+	  group->getSegmentMan()->getFastestPeerStats();
 	for(std::deque<SharedHandle<PeerStat> >::const_iterator i =
 	      peerStats.begin(); i != peerStats.end(); ++i) {
 	  if((*i)->getHostname().empty() || (*i)->getProtocol().empty()) {
@@ -357,7 +360,7 @@ public:
 						    (*i)->getProtocol());
           ss->increaseCounter();
 	  ss->updateDownloadSpeed(speed);
-          if(peerStats.size() == 1) {
+          if(singleConnection) {
             ss->updateSingleConnectionAvgSpeed(speed);
           }
           else {
@@ -392,10 +395,13 @@ void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
 		ProcessStoppedRequestGroup(e, _reservedGroups,
 					   _downloadResults));
 
-  _requestGroups.erase(std::remove_if(_requestGroups.begin(),
-				      _requestGroups.end(),
-				      FindStoppedRequestGroup()),
-		       _requestGroups.end());
+  std::deque<SharedHandle<RequestGroup> >::iterator i =
+    std::remove_if(_requestGroups.begin(),
+		   _requestGroups.end(),
+		   FindStoppedRequestGroup());
+  if(i != _requestGroups.end()) {
+    _requestGroups.erase(i, _requestGroups.end());
+  }
 
   size_t numRemoved = numPrev-_requestGroups.size();
   if(numRemoved > 0) {
@@ -423,13 +429,9 @@ void RequestGroupMan::configureRequestGroup
 
 static void createInitialCommand(const SharedHandle<RequestGroup>& requestGroup,
 				 std::deque<Command*>& commands,
-				 DownloadEngine* e,
-				 bool useHead)
+				 DownloadEngine* e)
 {
-  requestGroup->createInitialCommand(commands, e,
-				     useHead ?
-				     Request::METHOD_HEAD :
-				     Request::METHOD_GET);
+  requestGroup->createInitialCommand(commands, e);
 }
 
 void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
@@ -444,66 +446,47 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
   while(count < num && !_reservedGroups.empty()) {
     RequestGroupHandle groupToAdd = _reservedGroups.front();
     _reservedGroups.pop_front();
+    Commands commands;
     try {
       if(!groupToAdd->isDependencyResolved()) {
 	temp.push_back(groupToAdd);
 	continue;
       }
       configureRequestGroup(groupToAdd);
-      Commands commands;
-      createInitialCommand(groupToAdd, commands, e,
-			   _option->getAsBool(PREF_USE_HEAD)||
-			   _option->getAsBool(PREF_DRY_RUN));
+      createInitialCommand(groupToAdd, commands, e);
       _requestGroups.push_back(groupToAdd);
       ++count;
       e->addCommand(commands);
+      commands.clear();
       executeStartHook(groupToAdd, e->option);
     } catch(RecoverableException& ex) {
       _logger->error(EX_EXCEPTION_CAUGHT, ex);
+      _logger->debug("Deleting temporal commands.");
+      std::for_each(commands.begin(), commands.end(), Deleter());
+      commands.clear();
+      _logger->debug("Commands deleted");
       groupToAdd->releaseRuntimeResource(e);
       _downloadResults.push_back(groupToAdd->createDownloadResult());
     }
   }
-  _reservedGroups.insert(_reservedGroups.begin(), temp.begin(), temp.end());
+  if(!temp.empty()) {
+    _reservedGroups.insert(_reservedGroups.begin(), temp.begin(), temp.end());
+  }
   if(count > 0) {
     e->setNoWait(true);
     _logger->debug("%d RequestGroup(s) added.", count);
   }
 }
 
-void RequestGroupMan::getInitialCommands(std::deque<Command*>& commands,
-					 DownloadEngine* e)
-{
-  for(RequestGroups::iterator itr = _requestGroups.begin();
-	itr != _requestGroups.end();) {
-    try {
-      if((*itr)->isDependencyResolved()) {
-	configureRequestGroup(*itr);
-	createInitialCommand(*itr, commands, e,
-			     _option->getAsBool(PREF_USE_HEAD));
-	executeStartHook(*itr, e->option);
-	++itr;
-      } else {
-	_reservedGroups.push_front((*itr));
-	itr = _requestGroups.erase(itr);
-      }
-    } catch(RecoverableException& e) {
-      _logger->error(EX_EXCEPTION_CAUGHT, e);
-      _downloadResults.push_back((*itr)->createDownloadResult());
-      itr = _requestGroups.erase(itr);
-    }  
-  }
-}
-
 void RequestGroupMan::save()
 {
   for(RequestGroups::iterator itr = _requestGroups.begin();
       itr != _requestGroups.end(); ++itr) {
     if((*itr)->allDownloadFinished()) {
-      (*itr)->getProgressInfoFile()->removeFile();
+      (*itr)->removeControlFile();
     } else {
       try {
-	(*itr)->getProgressInfoFile()->save();
+	(*itr)->saveControlFile();
       } catch(RecoverableException& e) {
 	_logger->error(EX_EXCEPTION_CAUGHT, e);
       }
@@ -524,10 +507,10 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
   size_t finished = 0;
   size_t error = 0;
   size_t inprogress = 0;
-  DownloadResult::RESULT lastError = DownloadResult::FINISHED;
+  downloadresultcode::RESULT lastError = downloadresultcode::FINISHED;
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr = _downloadResults.begin();
       itr != _downloadResults.end(); ++itr) {
-    if((*itr)->result == DownloadResult::FINISHED) {
+    if((*itr)->result == downloadresultcode::FINISHED) {
       ++finished;
     } else {
       ++error;
@@ -537,7 +520,7 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
   for(RequestGroups::const_iterator itr = _requestGroups.begin();
       itr != _requestGroups.end(); ++itr) {
     DownloadResultHandle result = (*itr)->createDownloadResult();
-    if(result->result == DownloadResult::FINISHED) {
+    if(result->result == downloadresultcode::FINISHED) {
       ++finished;
     } else {
       ++inprogress;
@@ -559,7 +542,7 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
   o << "\n"
     <<_("Download Results:") << "\n"
     << "gid|stat|avg speed  |path/URI" << "\n"
-    << "===+====+===========+==========================================================" << "\n";
+    << "===+====+===========+===========================================================" << "\n";
 
   int ok = 0;
   int err = 0;
@@ -568,10 +551,10 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr = _downloadResults.begin();
       itr != _downloadResults.end(); ++itr) {
     std::string status;
-    if((*itr)->result == DownloadResult::FINISHED) {
+    if((*itr)->result == downloadresultcode::FINISHED) {
       status = MARK_OK;
       ++ok;
-    } else if((*itr)->result == DownloadResult::IN_PROGRESS) {
+    } else if((*itr)->result == downloadresultcode::IN_PROGRESS) {
       status = MARK_INPR;
       ++inpr;
     } else {
@@ -584,7 +567,7 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
       itr != _requestGroups.end(); ++itr) {
     DownloadResultHandle result = (*itr)->createDownloadResult();
     std::string status;
-    if(result->result == DownloadResult::FINISHED) {
+    if(result->result == downloadresultcode::FINISHED) {
       status = MARK_OK;
       ++ok;
     } else {
@@ -616,54 +599,6 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
   }
 }
 
-template<typename InputIterator>
-static SharedHandle<FileEntry> getFirstRequestedFileEntry
-(InputIterator first, InputIterator last)
-{
-  for(; first != last; ++first) {
-    if((*first)->isRequested()) {
-      return *first;
-    }
-  }
-  return SharedHandle<FileEntry>();
-}
-
-template<typename InputIterator>
-static size_t countRequestedFileEntry(InputIterator first, InputIterator last)
-{
-  size_t count = 0;
-  for(; first != last; ++first) {
-    if((*first)->isRequested()) {
-      ++count;
-    }
-  }
-  return count;
-}
-
-template<typename InputIterator>
-static void writeFilePath
-(InputIterator first, InputIterator last, std::ostream& o, bool memory)
-{
-  SharedHandle<FileEntry> e = getFirstRequestedFileEntry(first, last);
-  if(e.isNull()) {
-    o << "n/a";
-  } else {
-    if(e->getPath().empty()) {
-      o << "n/a";
-    } else {
-      if(memory) {
-	o << "[MEMORY]" << File(e->getPath()).getBasename();
-      } else {
-	o << e->getPath();
-      }
-    }
-    size_t count = countRequestedFileEntry(first, last);
-    if(count > 1) {
-      o << " (" << count-1 << "more)";
-    }
-  }
-}
-
 std::string RequestGroupMan::formatDownloadResult(const std::string& status, const DownloadResultHandle& downloadResult) const
 {
   std::stringstream o;
@@ -671,25 +606,17 @@ std::string RequestGroupMan::formatDownloadResult(const std::string& status, con
     << std::setw(4) << status << "|"
     << std::setw(11);
   if(downloadResult->sessionTime > 0) {
-    o << Util::abbrevSize
+    o << util::abbrevSize
       (downloadResult->sessionDownloadLength*1000/downloadResult->sessionTime)+
       "B/s";
   } else {
     o << "n/a";
   }
   o << "|";
-  const std::deque<SharedHandle<FileEntry> >& fileEntries =
+  const std::vector<SharedHandle<FileEntry> >& fileEntries =
     downloadResult->fileEntries;
-  if(downloadResult->result == DownloadResult::FINISHED ||
-     downloadResult->numUri == 0) {
-    writeFilePath(fileEntries.begin(), fileEntries.end(), o,
-		  downloadResult->inMemoryDownload);
-  } else {
-    o << downloadResult->uri;
-    if(downloadResult->numUri > 1) {
-      o << " (" << downloadResult->numUri-1 << "more)";
-    }
-  }
+  writeFilePath(fileEntries.begin(), fileEntries.end(), o,
+		downloadResult->inMemoryDownload);
   return o.str();
 }
 
@@ -718,7 +645,7 @@ bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) cons
   for(RequestGroups::const_iterator itr = _requestGroups.begin();
       itr != _requestGroups.end(); ++itr) {
     if((*itr).get() != requestGroup) {
-      std::deque<SharedHandle<FileEntry> > entries =
+      const std::vector<SharedHandle<FileEntry> >& entries =
 	(*itr)->getDownloadContext()->getFileEntries();
       std::transform(entries.begin(), entries.end(),
 		     std::back_inserter(files),
@@ -726,7 +653,7 @@ bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) cons
     }
   }
   std::sort(files.begin(), files.end());
-  std::deque<SharedHandle<FileEntry> > entries =
+  const std::vector<SharedHandle<FileEntry> >& entries =
     requestGroup->getDownloadContext()->getFileEntries();
   return sameFilePathExists(files.begin(), files.end(),
 			    entries.begin(), entries.end());
@@ -819,21 +746,25 @@ bool RequestGroupMan::saveServerStat(const std::string& filename) const
 {
   std::string tempfile = filename;
   tempfile += "__temp";
-  std::ofstream out(tempfile.c_str(), std::ios::binary);
-  if(!out) {
-    _logger->error(MSG_OPENING_WRITABLE_SERVER_STAT_FILE_FAILED,
-		   tempfile.c_str());
-    return false;
-  }
-  if (_serverStatMan->save(out)) {
-    out.close();
-    if (File(tempfile).renameTo(filename)) {
-      _logger->notice(MSG_SERVER_STAT_SAVED, filename.c_str());
-      return true;
+  {
+    std::ofstream out(tempfile.c_str(), std::ios::binary);
+    if(!out) {
+      _logger->error(MSG_OPENING_WRITABLE_SERVER_STAT_FILE_FAILED,
+		     filename.c_str());
+      return false;
+    }
+    if(!_serverStatMan->save(out)) {
+      _logger->error(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str());
+      return false;
     }
   }
-  _logger->error(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str());
-  return false;
+  if(File(tempfile).renameTo(filename)) {
+    _logger->notice(MSG_SERVER_STAT_SAVED, filename.c_str());
+    return true;
+  } else {
+    _logger->error(MSG_WRITING_SERVER_STAT_FILE_FAILED, filename.c_str());
+    return false;
+  }
 }
 
 void RequestGroupMan::removeStaleServerStat(time_t timeout)
