@@ -35,6 +35,9 @@
 #include "SocketCore.h"
 
 #include <unistd.h>
+#ifdef HAVE_IFADDRS_H
+# include <ifaddrs.h>
+#endif // HAVE_IFADDRS_H
 
 #include <cerrno>
 #include <cstring>
@@ -48,10 +51,10 @@
 #include "DlRetryEx.h"
 #include "DlAbortEx.h"
 #include "StringFormat.h"
-#include "Util.h"
-#include "LogFactory.h"
+#include "util.h"
 #include "TimeA2.h"
 #include "a2functional.h"
+#include "LogFactory.h"
 #ifdef ENABLE_SSL
 # include "TLSContext.h"
 #endif // ENABLE_SSL
@@ -76,6 +79,17 @@
 
 namespace aria2 {
 
+#ifdef HAVE_EPOLL
+SocketCore::PollMethod SocketCore::_pollMethod = SocketCore::POLL_METHOD_EPOLL;
+#else // !HAVE_EPOLL
+SocketCore::PollMethod SocketCore::_pollMethod = SocketCore::POLL_METHOD_SELECT;
+#endif // !HAVE_EPOLL
+
+int SocketCore::_protocolFamily = AF_UNSPEC;
+
+std::vector<std::pair<struct sockaddr_storage, socklen_t> >
+SocketCore::_bindAddrs;
+
 #ifdef ENABLE_SSL
 SharedHandle<TLSContext> SocketCore::_tlsContext;
 
@@ -152,50 +166,89 @@ std::string uitos(T value)
   return str;
 }
 
-void SocketCore::bind(uint16_t port)
+static sock_t bindInternal(int family, int socktype, int protocol,
+			   const struct sockaddr* addr, socklen_t addrlen)
+{
+  sock_t fd = socket(family, socktype, protocol);
+  if(fd == (sock_t) -1) {
+    return -1;
+  }
+  int sockopt = 1;
+  if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (a2_sockopt_t) &sockopt,
+		sizeof(sockopt)) < 0) {
+    CLOSE(fd);
+    return -1;
+  }
+  if(::bind(fd, addr, addrlen) == -1) {
+    LogFactory::getInstance()->debug(EX_SOCKET_BIND, strerror(errno));
+    CLOSE(fd);
+    return -1;
+  }
+  return fd;
+}
+
+void SocketCore::bind(uint16_t port, int flags)
 {
   closeConnection();
 
-  struct addrinfo hints;
-  struct addrinfo* res;
-  memset(&hints, 0, sizeof(hints));
-  hints.ai_family = AF_UNSPEC;
-  hints.ai_socktype = _sockType;
-  hints.ai_flags = AI_PASSIVE;
-  hints.ai_protocol = 0;
-  int s;
-  s = getaddrinfo(0, uitos(port).c_str(), &hints, &res);
-  if(s) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_BIND, gai_strerror(s)).str());
+  if(flags == 0 || _bindAddrs.empty()) {
+    struct addrinfo hints;
+    struct addrinfo* res;
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = _protocolFamily;
+    hints.ai_socktype = _sockType;
+    hints.ai_flags = flags;
+    hints.ai_protocol = 0;
+    int s;
+    s = getaddrinfo(0, uitos(port).c_str(), &hints, &res);
+    if(s) {
+      throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, gai_strerror(s)).str());
+    }
+    struct addrinfo* rp;
+    for(rp = res; rp; rp = rp->ai_next) {
+      sock_t fd = bindInternal(rp->ai_family, rp->ai_socktype, rp->ai_protocol,
+			       rp->ai_addr, rp->ai_addrlen);
+      if(fd == (sock_t) -1) {
+	continue;
+      }
+      sockfd = fd;
+      break;
+    }
+    freeaddrinfo(res);
+  } else {
+    for(std::vector<std::pair<struct sockaddr_storage, socklen_t> >::
+	  const_iterator i = _bindAddrs.begin(); i != _bindAddrs.end(); ++i) {
+      sock_t fd = bindInternal
+	((*i).first.ss_family, _sockType, 0,
+	 reinterpret_cast<const struct sockaddr*>
+	 (&(*i).first), (*i).second);
+      if(fd != (sock_t)-1) {
+	sockfd = fd;
+	break;
+      }
+    }
   }
-  struct addrinfo* rp;
-  for(rp = res; rp; rp = rp->ai_next) {
-    sock_t fd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
-    if(fd == -1) {
-      continue;
-    }
-    int sockopt = 1;
-    if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (a2_sockopt_t) &sockopt, sizeof(sockopt)) < 0) {
-      CLOSE(fd);
-      continue;
-    }
-    if(::bind(fd, rp->ai_addr, rp->ai_addrlen) == -1) {
-      CLOSE(fd);
-      continue;
-    }
+  if(sockfd == (sock_t) -1) {
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, strerror(errno)).str());
+  }
+}
+
+void SocketCore::bind(const struct sockaddr* addr, socklen_t addrlen)
+{
+  closeConnection();
+
+  sock_t fd = bindInternal(addr->sa_family, _sockType, 0, addr, addrlen);
+  if(fd != (sock_t)-1) {
     sockfd = fd;
-    break;
-  }
-  freeaddrinfo(res);
-  if(sockfd == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_BIND, "all addresses failed").str());
+  } else {
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, strerror(errno)).str());
   }
 }
 
 void SocketCore::beginListen()
 {
   if(listen(sockfd, 1) == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_LISTEN, errorMsg()).str());
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_LISTEN, errorMsg()).str());
   }
 }
 
@@ -204,9 +257,9 @@ SocketCore* SocketCore::acceptConnection() const
   struct sockaddr_storage sockaddr;
   socklen_t len = sizeof(sockaddr);
   sock_t fd;
-  while((fd = accept(sockfd, reinterpret_cast<struct sockaddr*>(&sockaddr), &len)) == -1 && SOCKET_ERRNO == EINTR);
-  if(fd == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_ACCEPT, errorMsg()).str());
+  while((fd = accept(sockfd, reinterpret_cast<struct sockaddr*>(&sockaddr), &len)) == (sock_t) -1 && SOCKET_ERRNO == EINTR);
+  if(fd == (sock_t) -1) {
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_ACCEPT, errorMsg()).str());
   }
   return new SocketCore(fd, _sockType);
 }
@@ -217,9 +270,9 @@ void SocketCore::getAddrInfo(std::pair<std::string, uint16_t>& addrinfo) const
   socklen_t len = sizeof(sockaddr);
   struct sockaddr* addrp = reinterpret_cast<struct sockaddr*>(&sockaddr);
   if(getsockname(sockfd, addrp, &len) == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
   }
-  addrinfo = Util::getNumericNameInfo(addrp, len);
+  addrinfo = util::getNumericNameInfo(addrp, len);
 }
 
 void SocketCore::getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const
@@ -228,9 +281,9 @@ void SocketCore::getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const
   socklen_t len = sizeof(sockaddr);
   struct sockaddr* addrp = reinterpret_cast<struct sockaddr*>(&sockaddr);
   if(getpeername(sockfd, addrp, &len) == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
   }
-  peerinfo = Util::getNumericNameInfo(addrp, len);
+  peerinfo = util::getNumericNameInfo(addrp, len);
 }
 
 void SocketCore::establishConnection(const std::string& host, uint16_t port)
@@ -240,20 +293,20 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
   struct addrinfo hints;
   struct addrinfo* res;
   memset(&hints, 0, sizeof(hints));
-  hints.ai_family = AF_UNSPEC;
+  hints.ai_family = _protocolFamily;
   hints.ai_socktype = _sockType;
   hints.ai_flags = 0;
   hints.ai_protocol = 0;
   int s;
   s = getaddrinfo(host.c_str(), uitos(port).c_str(), &hints, &res);
   if(s) {
-    throw DlAbortEx(StringFormat(EX_RESOLVE_HOSTNAME,
+    throw DL_ABORT_EX(StringFormat(EX_RESOLVE_HOSTNAME,
 				 host.c_str(), gai_strerror(s)).str());
   }
   struct addrinfo* rp;
   for(rp = res; rp; rp = rp->ai_next) {
     sock_t fd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
-    if(fd == -1) {
+    if(fd == (sock_t) -1) {
       continue;
     }
     int sockopt = 1;
@@ -261,13 +314,31 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
       CLOSE(fd);
       continue;
     }
+    if(!_bindAddrs.empty()) {
+      bool bindSuccess = false;
+      for(std::vector<std::pair<struct sockaddr_storage, socklen_t> >::
+	    const_iterator i = _bindAddrs.begin(); i != _bindAddrs.end(); ++i) {
+	if(::bind(fd,reinterpret_cast<const struct sockaddr*>(&(*i).first),
+		  (*i).second) == -1) {
+	  LogFactory::getInstance()->debug(EX_SOCKET_BIND, strerror(errno));
+	} else {
+	  bindSuccess = true;
+	  break;
+	}
+      }
+      if(!bindSuccess) {
+	CLOSE(fd);
+	continue;
+      }
+    }
+
     sockfd = fd;
     // make socket non-blocking mode
     setNonBlockingMode();
     if(connect(fd, rp->ai_addr, rp->ai_addrlen) == -1 &&
        SOCKET_ERRNO != A2_EINPROGRESS) {
       CLOSE(sockfd);
-      sockfd = -1;
+      sockfd = (sock_t) -1;
       continue;
     }
     // TODO at this point, connection may not be established and it may fail
@@ -275,9 +346,9 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
     break;
   }
   freeaddrinfo(res);
-  if(sockfd == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_CONNECT, host.c_str(),
-				 "all addresses failed").str());
+  if(sockfd == (sock_t) -1) {
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_CONNECT, host.c_str(),
+				   strerror(errno)).str());
   }
 }
 
@@ -286,7 +357,7 @@ void SocketCore::setNonBlockingMode()
 #ifdef __MINGW32__
   static u_long flag = 1;
   if (::ioctlsocket(sockfd, FIONBIO, &flag) == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_NONBLOCKING, errorMsg()).str());
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_NONBLOCKING, errorMsg()).str());
   }
 #else
   int flags;
@@ -302,7 +373,7 @@ void SocketCore::setBlockingMode()
 #ifdef __MINGW32__
   static u_long flag = 0;
   if (::ioctlsocket(sockfd, FIONBIO, &flag) == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_BLOCKING, errorMsg()).str());
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BLOCKING, errorMsg()).str());
   }
 #else
   int flags;
@@ -326,7 +397,7 @@ void SocketCore::closeConnection()
     gnutls_bye(sslSession, GNUTLS_SHUT_RDWR);
   }
 #endif // HAVE_LIBGNUTLS
-  if(sockfd != -1) {
+  if(sockfd != (sock_t) -1) {
     CLOSE(sockfd);
     sockfd = -1;
   }
@@ -348,7 +419,7 @@ void SocketCore::closeConnection()
 void SocketCore::initEPOLL()
 {
   if((_epfd = epoll_create(1)) == -1) {
-    throw DlRetryEx(StringFormat("epoll_create failed:%s", errorMsg()).str());
+    throw DL_RETRY_EX(StringFormat("epoll_create failed:%s", errorMsg()).str());
   }
 
   memset(&_epEvent, 0, sizeof(struct epoll_event));
@@ -356,7 +427,7 @@ void SocketCore::initEPOLL()
   _epEvent.data.fd = sockfd;
 
   if(epoll_ctl(_epfd, EPOLL_CTL_ADD, sockfd, &_epEvent) == -1) {
-    throw DlRetryEx(StringFormat("epoll_ctl failed:%s", errorMsg()).str());
+    throw DL_RETRY_EX(StringFormat("epoll_ctl failed:%s", errorMsg()).str());
   }
 }
 
@@ -364,50 +435,48 @@ void SocketCore::initEPOLL()
 
 bool SocketCore::isWritable(time_t timeout)
 {
-
 #ifdef HAVE_EPOLL
-
-  if(_epfd == -1) {
-    initEPOLL();
-  }
-  struct epoll_event epEvents[1];
-  int r;
-  while((r = epoll_wait(_epfd, epEvents, 1, 0)) == -1 && errno == EINTR);
-
-  if(r > 0) {
-    return epEvents[0].events&(EPOLLOUT|EPOLLHUP|EPOLLERR);
-  } else if(r == 0) {
-    return false;
-  } else {
-    throw DlRetryEx(StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
-  }
-
-#else // !HAVE_EPOLL
-
-  fd_set fds;
-  FD_ZERO(&fds);
-  FD_SET(sockfd, &fds);
-
-  struct timeval tv;
-  tv.tv_sec = timeout;
-  tv.tv_usec = 0;
-
-  int r = select(sockfd+1, NULL, &fds, NULL, &tv);
-  if(r == 1) {
-    return true;
-  } else if(r == 0) {
-    // time out
-    return false;
-  } else {
-    if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == EINTR) {
+  if(_pollMethod == SocketCore::POLL_METHOD_EPOLL) {
+    if(_epfd == -1) {
+      initEPOLL();
+    }
+    struct epoll_event epEvents[1];
+    int r;
+    while((r = epoll_wait(_epfd, epEvents, 1, 0)) == -1 && errno == EINTR);
+    if(r > 0) {
+      return epEvents[0].events&(EPOLLOUT|EPOLLHUP|EPOLLERR);
+    } else if(r == 0) {
       return false;
     } else {
-      throw DlRetryEx(StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
+      throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
     }
-  }
+  } else
+#endif // HAVE_EPOLL
+    if(_pollMethod == SocketCore::POLL_METHOD_SELECT) {
+      fd_set fds;
+      FD_ZERO(&fds);
+      FD_SET(sockfd, &fds);
 
-#endif // !HAVE_EPOLL
+      struct timeval tv;
+      tv.tv_sec = timeout;
+      tv.tv_usec = 0;
 
+      int r = select(sockfd+1, NULL, &fds, NULL, &tv);
+      if(r == 1) {
+	return true;
+      } else if(r == 0) {
+	// time out
+	return false;
+      } else {
+	if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == EINTR) {
+	  return false;
+	} else {
+	  throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
+	}
+      }
+    } else {
+      abort();
+    }
 }
 
 bool SocketCore::isReadable(time_t timeout)
@@ -419,48 +488,48 @@ bool SocketCore::isReadable(time_t timeout)
 #endif // HAVE_LIBGNUTLS
 
 #ifdef HAVE_EPOLL
+  if(_pollMethod == SocketCore::POLL_METHOD_EPOLL) {
+    if(_epfd == -1) {
+      initEPOLL();
+    }
+    struct epoll_event epEvents[1];
+    int r;
+    while((r = epoll_wait(_epfd, epEvents, 1, 0)) == -1 && errno == EINTR);
 
-  if(_epfd == -1) {
-    initEPOLL();
-  }
-  struct epoll_event epEvents[1];
-  int r;
-  while((r = epoll_wait(_epfd, epEvents, 1, 0)) == -1 && errno == EINTR);
-
-  if(r > 0) {
-    return epEvents[0].events&(EPOLLIN|EPOLLHUP|EPOLLERR);
-  } else if(r == 0) {
-    return false;
-  } else {
-    throw DlRetryEx(StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
-  }
-
-#else // !HAVE_EPOLL
-
-  fd_set fds;
-  FD_ZERO(&fds);
-  FD_SET(sockfd, &fds);
-
-  struct timeval tv;
-  tv.tv_sec = timeout;
-  tv.tv_usec = 0;
-
-  int r = select(sockfd+1, &fds, NULL, NULL, &tv);
-  if(r == 1) {
-    return true;
-  } else if(r == 0) {
-    // time out
-    return false;
-  } else {
-    if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == EINTR) {
+    if(r > 0) {
+      return epEvents[0].events&(EPOLLIN|EPOLLHUP|EPOLLERR);
+    } else if(r == 0) {
       return false;
     } else {
-      throw DlRetryEx(StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
+      throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
     }
-  }
+  } else
+#endif // HAVE_EPOLL
+    if(_pollMethod == SocketCore::POLL_METHOD_SELECT) {
+      fd_set fds;
+      FD_ZERO(&fds);
+      FD_SET(sockfd, &fds);
 
-#endif // !HAVE_EPOLL
+      struct timeval tv;
+      tv.tv_sec = timeout;
+      tv.tv_usec = 0;
 
+      int r = select(sockfd+1, &fds, NULL, NULL, &tv);
+      if(r == 1) {
+	return true;
+      } else if(r == 0) {
+	// time out
+	return false;
+      } else {
+	if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == EINTR) {
+	  return false;
+	} else {
+	  throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
+	}
+      }
+    } else {
+      abort();
+    }
 }
 
 #ifdef HAVE_LIBSSL
@@ -504,14 +573,14 @@ ssize_t SocketCore::writeData(const char* data, size_t len)
 	_wantWrite = true;
 	ret = 0;
       } else {
-	throw DlRetryEx(StringFormat(EX_SOCKET_SEND, errorMsg()).str());
+	throw DL_RETRY_EX(StringFormat(EX_SOCKET_SEND, errorMsg()).str());
       }
     }
   } else {
 #ifdef HAVE_LIBSSL
     ret = SSL_write(ssl, data, len);
     if(ret == 0) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat
 	 (EX_SOCKET_SEND, ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
     }
@@ -519,7 +588,7 @@ ssize_t SocketCore::writeData(const char* data, size_t len)
       ret = sslHandleEAGAIN(ret);
     }
     if(ret < 0) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat
 	 (EX_SOCKET_SEND, ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
     }
@@ -531,7 +600,7 @@ ssize_t SocketCore::writeData(const char* data, size_t len)
       gnutlsRecordCheckDirection();
       ret = 0;
     } else if(ret < 0) {
-      throw DlRetryEx(StringFormat(EX_SOCKET_SEND, gnutls_strerror(ret)).str());
+      throw DL_RETRY_EX(StringFormat(EX_SOCKET_SEND, gnutls_strerror(ret)).str());
     }
 #endif // HAVE_LIBGNUTLS
   }
@@ -553,7 +622,7 @@ void SocketCore::readData(char* data, size_t& len)
 	_wantRead = true;
 	ret = 0;
       } else {
-	throw DlRetryEx(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
+	throw DL_RETRY_EX(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
       }
     }
   } else {
@@ -562,7 +631,7 @@ void SocketCore::readData(char* data, size_t& len)
      // TODO handling len == 0 case required
     ret = SSL_read(ssl, data, len);
     if(ret == 0) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat
 	 (EX_SOCKET_RECV, ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
     }
@@ -570,7 +639,7 @@ void SocketCore::readData(char* data, size_t& len)
       ret = sslHandleEAGAIN(ret);
     }
     if(ret < 0) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat
 	 (EX_SOCKET_RECV, ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
     }
@@ -581,7 +650,7 @@ void SocketCore::readData(char* data, size_t& len)
       gnutlsRecordCheckDirection();
       ret = 0;
     } else if(ret < 0) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat(EX_SOCKET_RECV, gnutls_strerror(ret)).str());
     }
 #endif // HAVE_LIBGNUTLS
@@ -603,7 +672,7 @@ void SocketCore::peekData(char* data, size_t& len)
 	_wantRead = true;
 	ret = 0;
       } else {
-	throw DlRetryEx(StringFormat(EX_SOCKET_PEEK, errorMsg()).str());
+	throw DL_RETRY_EX(StringFormat(EX_SOCKET_PEEK, errorMsg()).str());
       }
     }
   } else {
@@ -611,9 +680,8 @@ void SocketCore::peekData(char* data, size_t& len)
      // for SSL
      // TODO handling len == 0 case required
     ret = SSL_peek(ssl, data, len);
-    LogFactory::getInstance()->debug("len = %d", ret);
     if(ret == 0) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat(EX_SOCKET_PEEK,
 		      ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
     }
@@ -621,7 +689,7 @@ void SocketCore::peekData(char* data, size_t& len)
       ret = sslHandleEAGAIN(ret);
     }
     if(ret < 0) {
-      throw DlRetryEx
+      throw DL_RETRY_EX
 	(StringFormat(EX_SOCKET_PEEK,
 		      ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
     }
@@ -632,7 +700,7 @@ void SocketCore::peekData(char* data, size_t& len)
       gnutlsRecordCheckDirection();
       ret = 0;
     } else if(ret < 0) {
-      throw DlRetryEx(StringFormat(EX_SOCKET_PEEK,
+      throw DL_RETRY_EX(StringFormat(EX_SOCKET_PEEK,
 				   gnutls_strerror(ret)).str());
     }
 #endif // HAVE_LIBGNUTLS
@@ -681,7 +749,7 @@ static ssize_t GNUTLS_RECORD_RECV_NO_INTERRUPT
   while((ret = gnutls_record_recv(sslSession, data, len)) ==
 	GNUTLS_E_INTERRUPTED);
   if(ret < 0 && ret != GNUTLS_E_AGAIN) {
-    throw DlRetryEx
+    throw DL_RETRY_EX
       (StringFormat(EX_SOCKET_RECV, gnutls_strerror(ret)).str());
   }
   return ret;
@@ -727,25 +795,29 @@ void SocketCore::prepareSecureConnection()
     // for SSL
     ssl = SSL_new(_tlsContext->getSSLCtx());
     if(!ssl) {
-      throw DlAbortEx
+      throw DL_ABORT_EX
 	(StringFormat(EX_SSL_INIT_FAILURE,
 		      ERR_error_string(ERR_get_error(), 0)).str());
     }
     if(SSL_set_fd(ssl, sockfd) == 0) {
-      throw DlAbortEx
+      throw DL_ABORT_EX
 	(StringFormat(EX_SSL_INIT_FAILURE,
 		      ERR_error_string(ERR_get_error(), 0)).str());
     }
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
-    const int cert_type_priority[3] = { GNUTLS_CRT_X509,
-					GNUTLS_CRT_OPENPGP, 0
-    };
-    // while we do not support X509 certificate, most web servers require
-    // X509 stuff.
+    int r;
     gnutls_init(&sslSession, GNUTLS_CLIENT);
-    gnutls_set_default_priority(sslSession);
-    gnutls_kx_set_priority(sslSession, cert_type_priority);
+    // It seems err is not error message, but the argument string
+    // which causes syntax error.
+    const char* err;
+    // Disables TLS1.1 here because there are servers that don't
+    // understand TLS1.1.
+    r = gnutls_priority_set_direct(sslSession, "NORMAL:!VERS-TLS1.1", &err);
+    if(r != GNUTLS_E_SUCCESS) {
+      throw DL_ABORT_EX
+	(StringFormat(EX_SSL_INIT_FAILURE, gnutls_strerror(r)).str());
+    }
     // put the x509 credentials to the current session
     gnutls_credentials_set(sslSession, GNUTLS_CRD_CERTIFICATE,
 			   _tlsContext->getCertCred());
@@ -778,19 +850,19 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         case SSL_ERROR_WANT_X509_LOOKUP:
         case SSL_ERROR_ZERO_RETURN:
           if (blocking) {
-            throw DlAbortEx
+            throw DL_ABORT_EX
 	      (StringFormat(EX_SSL_CONNECT_ERROR, ssl_error).str());
           }
           break;
 
         case SSL_ERROR_SYSCALL:
-          throw DlAbortEx(EX_SSL_IO_ERROR);
+          throw DL_ABORT_EX(EX_SSL_IO_ERROR);
 
         case SSL_ERROR_SSL:
-          throw DlAbortEx(EX_SSL_PROTOCOL_ERROR);
+          throw DL_ABORT_EX(EX_SSL_PROTOCOL_ERROR);
 
         default:
-          throw DlAbortEx
+          throw DL_ABORT_EX
 	    (StringFormat(EX_SSL_UNKNOWN_ERROR, ssl_error).str());
       }
     }
@@ -798,19 +870,19 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
       // verify peer
       X509* peerCert = SSL_get_peer_certificate(ssl);
       if(!peerCert) {
-	throw DlAbortEx(MSG_NO_CERT_FOUND);
+	throw DL_ABORT_EX(MSG_NO_CERT_FOUND);
       }
       auto_delete<X509*> certDeleter(peerCert, X509_free);
 
       long verifyResult = SSL_get_verify_result(ssl);
       if(verifyResult != X509_V_OK) {
-	throw DlAbortEx
+	throw DL_ABORT_EX
 	  (StringFormat(MSG_CERT_VERIFICATION_FAILED,
 			X509_verify_cert_error_string(verifyResult)).str());
       }
       X509_NAME* name = X509_get_subject_name(peerCert);
       if(!name) {
-	throw DlAbortEx("Could not get X509 name object from the certificate.");
+	throw DL_ABORT_EX("Could not get X509 name object from the certificate.");
       }
 
       bool hostnameOK = false;
@@ -834,7 +906,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
 	}
       }
       if(!hostnameOK) {
-	throw DlAbortEx(MSG_HOSTNAME_NOT_MATCH);
+	throw DL_ABORT_EX(MSG_HOSTNAME_NOT_MATCH);
       }
     }
 #endif // HAVE_LIBSSL
@@ -844,7 +916,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
       gnutlsRecordCheckDirection();
       return false;
     } else if(ret < 0) {
-      throw DlAbortEx
+      throw DL_ABORT_EX
 	(StringFormat(EX_SSL_INIT_FAILURE, gnutls_strerror(ret)).str());
     }
 
@@ -853,7 +925,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
       unsigned int status;
       ret = gnutls_certificate_verify_peers2(sslSession, &status);
       if(ret < 0) {
-	throw DlAbortEx
+	throw DL_ABORT_EX
 	  (StringFormat("gnutls_certificate_verify_peer2() failed. Cause: %s",
 			gnutls_strerror(ret)).str());
       }
@@ -869,27 +941,27 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
 	  errors += " `issuer is not known'";
 	}
 	if(!errors.empty()) {
-	  throw DlAbortEx
+	  throw DL_ABORT_EX
 	    (StringFormat(MSG_CERT_VERIFICATION_FAILED, errors.c_str()).str());
 	}
       }
       // certificate type: only X509 is allowed.
       if(gnutls_certificate_type_get(sslSession) != GNUTLS_CRT_X509) {
-	throw DlAbortEx("Certificate type is not X509.");
+	throw DL_ABORT_EX("Certificate type is not X509.");
       }
 
       unsigned int peerCertsLength;
       const gnutls_datum_t* peerCerts = gnutls_certificate_get_peers
 	(sslSession, &peerCertsLength);
       if(!peerCerts) {
-	throw DlAbortEx(MSG_NO_CERT_FOUND);
+	throw DL_ABORT_EX(MSG_NO_CERT_FOUND);
       }
       Time now;
       for(unsigned int i = 0; i < peerCertsLength; ++i) {
 	gnutls_x509_crt_t cert;
 	ret = gnutls_x509_crt_init(&cert);
 	if(ret < 0) {
-	  throw DlAbortEx
+	  throw DL_ABORT_EX
 	    (StringFormat("gnutls_x509_crt_init() failed. Cause: %s",
 			  gnutls_strerror(ret)).str());
 	}
@@ -897,28 +969,28 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
 	  (cert, gnutls_x509_crt_deinit);
 	ret = gnutls_x509_crt_import(cert, &peerCerts[i], GNUTLS_X509_FMT_DER);
 	if(ret < 0) {
-	  throw DlAbortEx
+	  throw DL_ABORT_EX
 	    (StringFormat("gnutls_x509_crt_import() failed. Cause: %s",
 			  gnutls_strerror(ret)).str());
 	}
 	if(i == 0) {
 	  if(!gnutls_x509_crt_check_hostname(cert, hostname.c_str())) {
-	    throw DlAbortEx(MSG_HOSTNAME_NOT_MATCH);
+	    throw DL_ABORT_EX(MSG_HOSTNAME_NOT_MATCH);
 	  }
 	}
 	time_t activationTime = gnutls_x509_crt_get_activation_time(cert);
 	if(activationTime == -1) {
-	  throw DlAbortEx("Could not get activation time from certificate.");
+	  throw DL_ABORT_EX("Could not get activation time from certificate.");
 	}
 	if(now.getTime() < activationTime) {
-	  throw DlAbortEx("Certificate is not activated yet.");
+	  throw DL_ABORT_EX("Certificate is not activated yet.");
 	}
 	time_t expirationTime = gnutls_x509_crt_get_expiration_time(cert);
 	if(expirationTime == -1) {
-	  throw DlAbortEx("Could not get expiration time from certificate.");
+	  throw DL_ABORT_EX("Could not get expiration time from certificate.");
 	}
 	if(expirationTime < now.getTime()) {
-	  throw DlAbortEx("Certificate has expired.");
+	  throw DL_ABORT_EX("Certificate has expired.");
 	}
       }
     }
@@ -972,14 +1044,14 @@ ssize_t SocketCore::writeData(const char* data, size_t len,
   struct addrinfo hints;
   struct addrinfo* res;
   memset(&hints, 0, sizeof(hints));
-  hints.ai_family = AF_UNSPEC;
+  hints.ai_family = _protocolFamily;
   hints.ai_socktype = _sockType;
   hints.ai_flags = 0;
   hints.ai_protocol = 0;
   int s;
   s = getaddrinfo(host.c_str(), uitos(port).c_str(), &hints, &res);
   if(s) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_SEND, gai_strerror(s)).str());
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_SEND, gai_strerror(s)).str());
   }
   struct addrinfo* rp;
   ssize_t r = -1;
@@ -996,7 +1068,7 @@ ssize_t SocketCore::writeData(const char* data, size_t len,
   }
   freeaddrinfo(res);
   if(r == -1) {
-    throw DlAbortEx(StringFormat(EX_SOCKET_SEND, errorMsg()).str());
+    throw DL_ABORT_EX(StringFormat(EX_SOCKET_SEND, errorMsg()).str());
   }
   return r;
 }
@@ -1018,10 +1090,10 @@ ssize_t SocketCore::readDataFrom(char* data, size_t len,
       _wantRead = true;
       r = 0;
     } else {
-      throw DlRetryEx(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
+      throw DL_RETRY_EX(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
     }
   } else {
-    sender = Util::getNumericNameInfo(addrp, sockaddrlen);
+    sender = util::getNumericNameInfo(addrp, sockaddrlen);
   }
 
   return r;
@@ -1033,7 +1105,7 @@ std::string SocketCore::getSocketError() const
   socklen_t optlen = sizeof(error);
 
   if(getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (a2_sockopt_t) &error, &optlen) == -1) {
-    throw DlAbortEx(StringFormat("Failed to get socket error: %s",
+    throw DL_ABORT_EX(StringFormat("Failed to get socket error: %s",
 				 errorMsg()).str());
   }
   if(error != 0) {
@@ -1053,4 +1125,117 @@ bool SocketCore::wantWrite() const
   return _wantWrite;
 }
 
+#ifdef HAVE_EPOLL
+void SocketCore::useEpoll()
+{
+  _pollMethod = SocketCore::POLL_METHOD_EPOLL;
+}
+#endif // HAVE_EPOLL
+
+void SocketCore::useSelect()
+{
+  _pollMethod = SocketCore::POLL_METHOD_SELECT;
+}
+
+void SocketCore::bindAddress(const std::string& interface)
+{
+  std::vector<std::pair<struct sockaddr_storage, socklen_t> > bindAddrs;
+  LogFactory::getInstance()->debug("Finding interface %s", interface.c_str());
+#ifdef HAVE_GETIFADDRS
+  // First find interface in interface addresses
+  struct ifaddrs* ifaddr = 0;
+  if(getifaddrs(&ifaddr) == -1) {
+    throw DL_ABORT_EX
+      (StringFormat(MSG_INTERFACE_NOT_FOUND,
+		    interface.c_str(), strerror(errno)).str());
+  } else {
+    auto_delete<struct ifaddrs*> ifaddrDeleter(ifaddr, freeifaddrs);
+    for(struct ifaddrs* ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
+      if(!ifa->ifa_addr) {
+	continue;
+      }
+      int family = ifa->ifa_addr->sa_family;
+      if(_protocolFamily == AF_UNSPEC) {
+	if(family != AF_INET && family != AF_INET6) {
+	  continue;
+	}
+      } else if(_protocolFamily == AF_INET) {
+	if(family != AF_INET) {
+	  continue;
+	}
+      } else if(_protocolFamily == AF_INET6) {
+	if(family != AF_INET6) {
+	  continue;
+	}
+      } else {
+	continue;
+      }
+      if(std::string(ifa->ifa_name) == interface) {
+	socklen_t bindAddrLen =
+	  family == AF_INET?sizeof(struct sockaddr_in):
+	  sizeof(struct sockaddr_in6);
+	struct sockaddr_storage bindAddr;
+	memset(&bindAddr, 0, sizeof(bindAddr));
+	memcpy(&bindAddr, ifa->ifa_addr, bindAddrLen);
+	bindAddrs.push_back(std::make_pair(bindAddr, bindAddrLen));
+      }
+    }
+  }
+#endif // HAVE_GETIFADDRS
+  if(bindAddrs.empty()) {
+    struct addrinfo hints;
+    struct addrinfo* res = 0;
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = _protocolFamily;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = 0;
+    hints.ai_protocol = 0;
+    int s;
+    s = getaddrinfo(interface.c_str(), 0, &hints, &res);
+    if(s) {
+      throw DL_ABORT_EX
+	(StringFormat(MSG_INTERFACE_NOT_FOUND,
+		      interface.c_str(), gai_strerror(s)).str());
+    } else {
+      auto_delete<struct addrinfo*> resDeleter(res, freeaddrinfo);
+      struct addrinfo* rp;
+      for(rp = res; rp; rp = rp->ai_next) {
+	socklen_t bindAddrLen = rp->ai_addrlen;
+	struct sockaddr_storage bindAddr;
+	memset(&bindAddr, 0, sizeof(bindAddr));
+	memcpy(&bindAddr, rp->ai_addr, rp->ai_addrlen);
+	// Try to bind socket with this address. If it fails, the
+	// address is not for this machine.
+	try {
+	  SocketCore socket;
+	  socket.bind
+	    (reinterpret_cast<const struct sockaddr*>(&bindAddr), bindAddrLen);
+	  bindAddrs.push_back(std::make_pair(bindAddr, bindAddrLen));
+	} catch(RecoverableException& e) {
+	  continue;
+	}
+      }
+    }
+  }
+  if(bindAddrs.empty()) {
+      throw DL_ABORT_EX
+	(StringFormat(MSG_INTERFACE_NOT_FOUND,
+		      interface.c_str(), "not available").str());
+  } else {
+    _bindAddrs = bindAddrs;
+    for(std::vector<std::pair<struct sockaddr_storage, socklen_t> >::
+	  const_iterator i = _bindAddrs.begin(); i != _bindAddrs.end(); ++i) {
+      char host[NI_MAXHOST];
+      int s;
+      s = getnameinfo(reinterpret_cast<const struct sockaddr*>(&(*i).first),
+		      (*i).second,
+		      host, NI_MAXHOST, 0, NI_MAXSERV,
+		      NI_NUMERICHOST);
+      if(s == 0) {
+	LogFactory::getInstance()->debug("Sockets will bind to %s", host);
+      }
+    }
+  }
+}
+
 } // namespace aria2
