@@ -50,12 +50,12 @@
 #endif // !__CYGWIN__
 
 #ifdef __MINGW32__
-# define SOCKOPT_T const char
+# define a2_sockopt_t char *
 # define HAVE_GETADDRINFO
 # undef HAVE_GAI_STRERROR
 # undef gai_strerror
 #else
-# define SOCKOPT_T socklen_t
+# define a2_sockopt_t int *
 #endif // __MINGW32__
 
 #ifdef HAVE_NETDB_H
@@ -91,4 +91,10 @@
 # include "gai_strerror.h"
 #endif // HAVE_GAI_STRERROR
 
+#ifdef HAVE_WINSOCK2_H
+# define sock_t SOCKET
+#else
+# define sock_t int
+#endif
+
 #endif // _D_A2NETCOMPAT_H_
