@@ -41,7 +41,10 @@ _(" -o, --out=FILE               The file name of the downloaded file.")
 _(" -l, --log=LOG                The file name of the log file. If '-' is\n"\
   "                              specified, log is written to stdout.")
 #define TEXT_DAEMON \
-_(" -D, --daemon                 Run as daemon.")
+_(" -D, --daemon                 Run as daemon. The current working directory will\n"\
+  "                              be changed to \"/\" and standard input, standard\n"\
+  "                              output and standard error will be redirected to\n"\
+  "                              \"/dev/null\".")
 #define TEXT_SPLIT \
 _(" -s, --split=N                Download a file using N connections. If more\n"\
   "                              than N URLs are given, first N URLs are used and\n"\
@@ -83,9 +86,6 @@ _(" --http-user=USER             Set HTTP user. This affects all URLs.")
 _(" --http-passwd=PASSWD         Set HTTP password. This affects all URLs.")
 #define TEXT_PROXY_METHOD \
 _(" --proxy-method=METHOD        Set the method to use in proxy request.")
-#define TEXT_HTTP_AUTH_SCHEME \
-_(" --http-auth-scheme=SCHEME    Set HTTP authentication scheme. Currently, basic\n"\
-  "                              is the only supported scheme.")
 #define TEXT_REFERER \
 _(" --referer=REFERER            Set Referer. This affects all URLs.")
 #define TEXT_FTP_USER \
@@ -282,12 +282,11 @@ _(" --seed-ratio=RATIO           Specify share ratio. Seed completed torrents\n"
   "                              this option, seeding ends when at least one of\n"\
   "                              the conditions is satisfied.")
 #define TEXT_PEER_ID_PREFIX \
-_(" --peer-id-prefix=PEERI_ID_PREFIX Specify the prefix of peer ID. The peer ID in\n"\
+_(" --peer-id-prefix=PEER_ID_PREFIX Specify the prefix of peer ID. The peer ID in\n"\
   "                              BitTorrent is 20 byte length. If more than 20\n"\
-  "                              bytes are specified, only first 20\n"\
-  "                              bytes are used. If less than 20 bytes are\n"\
-  "                              specified, the random alphabet characters are\n"\
-  "                              added to make it's length 20 bytes.")
+  "                              bytes are specified, only first 20 bytes are\n"\
+  "                              used. If less than 20 bytes are specified, random\n"\
+  "                              byte data are added to make its length 20 bytes.")
 #define TEXT_ENABLE_PEER_EXCHANGE \
 _(" --enable-peer-exchange[=true|false] Enable Peer Exchange extension.")
 #define TEXT_ENABLE_DHT \
@@ -357,7 +356,7 @@ _(" --metalink-preferred-protocol=PROTO Specify preferred protocol. Specify 'non
   "                              if you don't have any preferred protocol.")
 #define TEXT_FOLLOW_METALINK \
 _(" --follow-metalink=true|false|mem If true or mem is specified, when a file\n"\
-  "                              whose suffix is .metaink or content type of\n"\
+  "                              whose suffix is .metalink or content type of\n"\
   "                              application/metalink+xml is downloaded, aria2\n"\
   "                              parses it as a metalink file and downloads files\n"\
   "                              mentioned in it.\n"\
@@ -374,13 +373,12 @@ _(" --metalink-enable-unique-protocol=true|false If true is given and several\n"
 #define TEXT_VERSION \
 _(" -v, --version                Print the version number and exit.")
 #define TEXT_HELP \
-_(" -h, --help[=CATEGORY]        Print usage and exit.\n"\
-  "                              The help messages are classified in several\n"\
-  "                              categories. For example, type \"--help=http\" for\n"\
-  "                              detailed explanation for the options related to\n"\
-  "                              http. If no matching category is found, search\n"\
-  "                              option name using a given word in middle match\n"\
-  "                              and print the result.")
+_(" -h, --help[=TAG|KEYWORD]     Print usage and exit.\n"\
+  "                              The help messages are classified with tags. A tag\n"\
+  "                              starts with \"#\". For example, type \"--help=#http\"\n"\
+  "                              to get the usage for the options tagged with\n"\
+  "                              \"#http\". If non-tag word is given, print the usage\n"\
+  "                              for the options whose name includes that word.")
 #define TEXT_NO_CONF \
 _(" --no-conf                    Disable loading aria2.conf file.")
 #define TEXT_CONF_PATH \
@@ -558,3 +556,28 @@ _(" --on-download-stop=COMMAND   Set the command to be executed when download\n"
   "                              option is not executed.\n"\
   "                              See --on-download-start option for the\n"\
   "                              requirement of COMMAND.")
+#define TEXT_BT_STOP_TIMEOUT \
+_(" --bt-stop-timeout=SEC        Stop BitTorrent download if download speed is 0 in\n"\
+  "                              consecutive SEC seconds. If 0 is given, this\n"\
+  "                              feature is disabled.")
+#define TEXT_XML_RPC_LISTEN_ALL \
+_(" --xml-rpc-listen-all[=true|false] Listen incoming XML-RPC requests on all\n"\
+  "                              network interfaces. If false is given, listen only\n"\
+  "                              on local loopback interface.")
+#define TEXT_BT_PRIORITIZE_PIECE \
+_(" --bt-prioritize-piece=head[=SIZE],tail[=SIZE] Try to download first and last\n"\
+  "                              pieces of each file first. This is useful for\n"\
+  "                              previewing files. The argument can contain 2\n"\
+  "                              keywords:head and tail. To include both keywords,\n"\
+  "                              they must be separated by comma. These keywords\n"\
+  "                              can take one parameter, SIZE. For example, if\n"\
+  "                              head=SIZE is specified, pieces in the range of\n"\
+  "                              first SIZE bytes of each file get higher priority.\n"\
+  "                              tail=SIZE means the range of last SIZE bytes of\n"\
+  "                              each file. SIZE can include K or M(1K = 1024, 1M =\n"\
+  "                              1024K). If SIZE is omitted, SIZE=1M is used.")
+#define TEXT_INTERFACE \
+_(" --interface=INTERFACE        Bind sockets to given interface. You can specify\n"\
+  "                              interface name, IP address and hostname.")
+#define TEXT_DISABLE_IPV6 \
+_(" --disable-ipv6[=true|false]  Disable IPv6.")
